% Funcin de filtrado de Savitzky-Golay
% Devuelve el valor filtrado (g) y el error
% cuadrtico medio (ecm).
% g=sumatorio(k=-mi,md)[c_k*y_(k+i)]
% donde c=M*(M'*M)^-1*e1 si o(1)=='n'
% o usando factorizacin QR si o(1)=='q'
%
% Uso: [g,ecm] = savigola(x,y,mi,md,n,o)
%  x = puntos de la funcin
%  y = valores de la funcin en el punto
%  mi = extremo izquierdo
%  md = extremo derecho
%  n = grado del polinomio de ajuste
%  o = 'ab' donde a puede ser 'n' (ecuaciones normales)
%      o 'q' (factorizacin QR) y b 'c' (filtrado de
%      los extremos) o 'n' (no filtra extremos)
%

function [g,ecm] = savigola(x,y,mI,mD,n,o)
     if nargin < 4
          error('Faltan argumentos de entrada');
     elseif nargin < 5
          o = 'qc';
     end;
     M = ones(mI+mD+1,n+1); N = length(y);
     for j=2:n+1
          M(:,j) = [-mI:mD]' .* M(:,j-1);
     end;
     if o(1)=='n'
          c = M*inv(M'*M)*[1;zeros(n,1)];
     elseif o(1)=='q'
          [Q,R] = qr(fliplr(M));
          c = (1/R(n+1,n+1))*Q(:,n+1);
     else
          error('Metodo desconocido para el problema minimo-cuadratico');
     end;
     for i=mI+1:N-mD
          g(i) = y(i-mI:mD+i)*c;
     end;
     if o(2) == 'n'
          g(1:mI) = y(1:mI);
          g(N-mD+1:N) = y(N-mD+1:N);
          ecm = norm(y-g,2)/(N-mI-mD);
     elseif o(2) == 'c'
          g(1:mI) = polyval(polyfit(x(1:mI),y(1:mI),n),x(1:mI));
          g(N-mD+1:N) = polyval(polyfit(x(N-mD+1:N),y(N-mD+1:N),n),x(N-mD+1:N));
          ecm = norm(y-g,2)/N;
     else
          error('Gestion de valores extremos desconocida');
     end;
