% Implementacion de metodos multipaso para
% Ecuaciones Diferenciales Ordinarias
% Metodo de Adams-Moulton de tercer orden
%
% Uso: [ts,ys]=admoul3(funcion,inicio,fin,alfa,paso,metodo)
%
% 'metodo' indica el metodo unipaso a emplear para el calculo
% de los valores iniciales que ademas se usan como valores
% predecidos. Por defecto es 'Runge-Kutta4'.
% Opciones: 'Runge-Kutta4', 'Pmedio', 'Euler-modificado',
% 'Heun', 'Kutta3', 'Nystrom', 'Kutta4', 'RK56A' y 'RK56B'
%
% (C) - Carlos Garcia Argos 20/VIII/2001

function [ts,ys]=admoul3(f,inicio,fin,alfa,h,metodo)
if nargin<5
   disp('Faltan argumentos de entrada');
   return;
else
   if nargin<6
      metodo='Runge-Kutta4';
   end;
   ts=inicio:h:fin;
   ys(:,1)=alfa;
   n=(fin-inicio)/h;
   % Clculo de los valores iniciales
   if strcmp(metodo,'Runge-Kutta4')
      [tpred,ypred]=rkc4(f,inicio,fin,alfa,h);
   elseif strcmp(metodo,'Pmedio')
      [tpred,ypred]=edpmedio(f,inicio,fin,alfa,h);
   elseif strcmp(metodo,'Euler-modificado')
      [tpred,ypred]=eulermod(f,inicio,fin,alfa,h);
   elseif strcmp(metodo,'Heun')
      [tpred,ypred]=heun(f,inicio,fin,alfa,h);
   elseif strcmp(metodo,'Kutta3')
      [tpred,ypred]=kutta3(f,inicio,fin,alfa,h);
   elseif strcmp(metodo,'Nystrom')
      [tpred,ypred]=nystrom(f,inicio,fin,alfa,h);
   elseif strcmp(metodo,'Kutta4')
      [tpred,ypred]=kutta4(f,inicio,fin,alfa,h);
   elseif strcmp(metodo,'RK56A')
      [tpred,ypred]=rk56a(f,inicio,fin,alfa,h);
   elseif strcmp(metodo,'RK56B')
      [tpred,ypred]=rk56b(f,inicio,fin,alfa,h);
   else
      disp('Metodo incorrecto');
      return;
   end;
   ys(:,2)=ypred(:,2);
   ys(:,3)=ypred(:,3);

   % Clculo de los restantes valores
   for i=2:n
      ys(:,i+1)=ys(:,i)+h*(5*feval(f,tpred(i+1),ypred(:,i+1))+8*feval(f,ts(i),ys(:,i))-feval(f,ts(i-1),ys(:,i-1)))/12;
   end;
end;
