% Implementacion de metodos multipaso para
% Ecuaciones Diferenciales Ordinarias
% Metodo de Hamming de cuarto orden con prediccion-correccion
%
% Uso: [ts,ys]=hamming(funcion,inicio,fin,alfa,paso,metodo)
%
% 'metodo' indica el metodo unipaso a emplear para el calculo
% de los valores iniciales. Por defecto es 'Runge-Kutta4'.
% Opciones: 'Runge-Kutta4', 'Pmedio', 'Euler-modificado',
% 'Heun', 'Kutta3', 'Nystrom', 'Kutta4', 'RK56A' y 'RK56B'
%
% (C) - Carlos Garcia Argos 20/VIII/2001

function [ts,ys]=hamming(f,inicio,fin,alfa,h,metodo)
if nargin<5
   disp('Faltan argumentos de entrada');
   return;
else
   if nargin<6
      metodo='Runge-Kutta4';
   end;
   ts=inicio:h:fin;
   ys(:,1)=alfa;
   n=(fin-inicio)/h;
   % Clculo de los valores iniciales
   if strcmp(metodo,'Runge-Kutta4')
      [tinic,yinic]=rkc4(f,inicio,inicio+3*h,alfa,h);
   elseif strcmp(metodo,'Pmedio')
      [tinic,yinic]=edpmedio(f,inicio,inicio+3*h,alfa,h);
   elseif strcmp(metodo,'Euler-modificado')
      [tinic,yinic]=eulermod(f,inicio,inicio+3*h,alfa,h);
   elseif strcmp(metodo,'Heun')
      [tinic,yinic]=heun(f,inicio,inicio+3*h,alfa,h);
   elseif strcmp(metodo,'Kutta3')
      [tinic,yinic]=kutta3(f,inicio,inicio+3*h,alfa,h);
   elseif strcmp(metodo,'Nystrom')
      [tinic,yinic]=nystrom(f,inicio,inicio+3*h,alfa,h);
   elseif strcmp(metodo,'Kutta4')
      [tinic,yinic]=kutta4(f,inicio,inicio+3*h,alfa,h);
   elseif strcmp(metodo,'RK56A')
      [tinic,yinic]=rk56a(f,inicio,inicio+3*h,alfa,h);
   elseif strcmp(metodo,'RK56B')
      [tinic,yinic]=rk56b(f,inicio,inicio+3*h,alfa,h);
   else
      disp('Metodo incorrecto');
      return;
   end;

   ys(:,2)=yinic(:,2);
   ys(:,3)=yinic(:,3);
   ys(:,4)=yinic(:,4);
   yc=ys;

   % Clculo de los restantes valores
   for i=4:n
      % Prediccion
      yp(:,i+1)=ys(:,i-3)+h*(8*feval(f,ts(i),ys(:,i))-4*feval(f,ts(i-1),ys(:,i-1))+8*feval(f,ts(i-2),ys(:,i-2)))/3;
      ym(:,i+1)=yp(:,i+1)-112*(yp(:,i)-yc(:,i))/121;
      % Correccion
      yc(:,i+1)=9/8*ys(:,i)-1/8*ys(:,i-2)+3*h*(feval(f,ts(i+1),ym(:,i+1))+2*feval(f,ts(i),ys(:,i))-feval(f,ts(i-1),ys(:,i-1)))/8;
      % Solucion
      ys(:,i+1)=yc(:,i+1)+9/121*(yp(:,i+1)-yc(:,i+1));
   end;
end;
