% Metodo de Newton para resolucion de
% ecuaciones no lineales
%
% Uso: [sol,error]=enewton(funcion,xinicial,maxiter,tol,metodo)
%
% metodo es el metodo de derivacion numerica que se usara.
% Las opciones son: deriv2p, derivp3p, derivr3p, derivp3p, derivp5p
% Por defecto es deriv2p, pero se puede especificar una funcion
% que contenga la derivada funcional.
% Si se especifica una funcion tendra nombre diferente a los metodos
% La tolerancia por defecto es tol=1e-5
%
% (C) - Carlos Garcia Argos, 7/VI/2000

function [xs,errs]=enewton(f,x0,maxiter,tol,metodo)

if nargin<5
   metodo='deriv2p';
   if nargin<4
      tol=1e-5;
   end;
   if nargin<3
      disp('Error: faltan argumentos de entrada');
      return;
   end;
end;

h=0.01;
iteraciones=0;
fin=0;
x(1)=x0;

while (iteraciones<maxiter)&(fin==0)
   iteraciones=iteraciones+1;
   fx=feval(f,x(iteraciones));
   fp=feval(metodo,f,x(iteraciones),h);
   x(iteraciones+1)=x(iteraciones)-(fx/fp);
   errs=abs(x(iteraciones+1)-x(iteraciones));
   if errs<tol
     fin=1;
   end;
end;

if fin==0
   disp('Se ha llegado al numero maximo de iteraciones');
else
   disp('Se ha encontrado solucion que cumple la tolerancia');
end;

xs=x(iteraciones+1);
