% Metodo de Laguerre para ecuaciones algebraicas
%
% Uso: [sol,error]=laguerre(pol,xinicial,maxiter,tol)
%
% La tolerancia por defecto es tol=1e-5
%
% (C) - Carlos Garcia Argos, 18/V/2003

function [xs,errs]=laguerre(poli,x0,maxiter,tol)

if nargin<4
   tol=1e-5;
end;
if nargin<3
   disp('Error: faltan argumentos de entrada');
   return;
end;

iteraciones=0;
fin=0;
x(1)=x0;
n = length(poli) - 1;

while (iteraciones<maxiter)&(fin==0)
   iteraciones=iteraciones+1;
   xant = x(iteraciones);
   num = polyval(poli,xant)*n;
   ppri = polyder(poli);
   pseg = polyder(ppri);
   poliev = polyval(poli,xant);
   ppriev = polyval(ppri,xant);
   psegev = polyval(pseg,xant);
   H = (n-1)*((n-1)*ppriev^2-n*poliev*psegev);
   denom = ppriev+sign(ppriev)*sqrt(H);
   x(iteraciones+1)=xant-num/denom;
   errs=abs(x(iteraciones+1)-x(iteraciones));
   if errs<tol
     fin=1;
   end;
end;

if fin==0
   disp('Se ha llegado al numero maximo de iteraciones');
else
   disp('Se ha encontrado solucion que cumple la tolerancia');
end;

xs=x(iteraciones+1);
