% Metodo de la regula falsi modificada para resolucion de
% ecuaciones no lineales de una sola variable
% Se selecciona mtodo de Pegasus o Illinois
% Uso:
%  sol=rfalsi(funcion,extizq,extder,maxiter,metodo,tol,error)
%
% Metodo puede ser 'Pegasus' o 'Illinois'
%
% El error es el valor maximo de diferencia entre el valor de la
% funcion en el punto y 0.
%
% (C) - Carlos Garcia Argos, 9/V/2003

function xs=rfalsimod(f,a,b,iter,metodo,tol,error)

if nargin<7
   error=0;
elseif nargin<6
   tol=0;
   if nargin<5
      metodo = 'Pegasus';
   end;

   if nargin<4
      disp('Error: faltan argumentos de entrada');
      return;
   end;
end;

% Pasamos a dividir sucesivamente el intervalo para buscar la solucion
fin=0;
iteracion=0;
fizq=feval(f,a);
fder=feval(f,b);
fant=fizq;

if fizq*fder<0
   while (iteracion<iter) & (abs(b-a)>tol) & (fin==0)
      c=(b*fizq-a*fder)/(fizq-fder);
      fcent=feval(f,c);
      if strcmp(metodo,'Pegasus')==1,
         alfa = fant/(fant+fcent);
      else
         if strcmp(metodo,'Illinois')==1,
            alfa = 0.5;
         else
            error('El metodo seleccionado es desconocido');
	 end;
      end;
      if (abs(fizq)>error)&(abs(fder)>error)
         if fizq*fcent<0
            b=c;
            fder=fcent;
	    if fcent*fant>0,
	       fizq = alfa*fizq;
	    end;
         elseif (fcent*fder<0)
            a=c;
            fizq=fcent;
	    if fcent*fant>0,
	       fder = alfa*fder;
	    end;
         end;
      else
         if abs(fizq)<=error
            xs=a;
         else
            xs=b;
         end;
         fin=1;
      end;
      iteracion=iteracion+1;
   end;
   if iteracion>=iter
      disp('Se ha llegado al numero maximo de iteraciones');
      xs=[a b];
   else
      if (abs(fcent)<error)|(a==b)
         disp('Se ha encontrado una solucion exacta');
         xs=c;
      else
         disp('Se ha encontrado un intervalo que cumple la tolerancia');
         xs=[a b];
      end;
   end;
else
   if fder==0
      xs=b;
   elseif fizq==0
      xs=a;
   else
      disp('Error: no existe ninguna solucion en el intervalo');
   end;
end;
