% Metodo de la secante para resolucion de
% ecuaciones no lineales de una sola variable
%
% Uso: sol=secante(funcion,xinicial,maxiter,tol)
%
% (C) - Carlos Garcia Argos, 7/VI/2000

function [xs,errs]=secante(f,x0,maxiter,tol)

if nargin<4
   tol=1e-5;
   if nargin<3
      disp('Error: faltan argumentos de entrada');
      return;
   end;
end;

% Obtener el primer punto con el metodo de Newton
iteraciones=0;
fin=0;
fx=feval(f,x0);
fp=feval('deriv2p',f,x0,0.01);
x(1)=x0;
x(2)=x(1)-(fx/fp);
iteraciones=iteraciones+1;

while (iteraciones<maxiter)&(fin==0)
   iteraciones=iteraciones+1;
   fx=feval(f,x(iteraciones));
   fp=fx-feval(f,x(iteraciones-1));
   x(iteraciones+1)=x(iteraciones)-(fx*(x(iteraciones)-x(iteraciones-1)))/fp;
   errs=abs(x(iteraciones+1)-x(iteraciones));
   if errs<tol
     fin=1;
   end;
end;

if fin==0
   disp('Se ha llegado al numero maximo de iteraciones');
else
   disp('Se ha encontrado solucion que cumple la tolerancia');
end;

xs=x(iteraciones+1);
