% Algoritmo para implementar la integracin adaptativa
% con cualquier mtodo de integracin (compuesta+simple)
%
% Uso:
%  sol = intadapt(funcion,a,b,tolerancia,metodo,maxiter)
%
%  metodo puede ser: Simpson, Trapecio, Simpson38, Boole,
%  Pmedio, Abierta2, Abierta3, Abierta4
% Si no se especifica metodo, aplica Simpson por defecto
% Si no se indica tolerancia se usa 1e-5
% Con maxiter se evita hacer demasiadas recursiones
%
% (C) - Carlos Garcia Argos, 20/IV/2003
%

function sol = intadapt(f,a,b,tol,metodo,maxiter)

if nargin < 6, maxiter = 10000; end;
if nargin < 5, metodo = 'Simpson'; end;
if nargin < 4, tol = 1e-5; end;
if nargin < 3, error('Faltan argumentos de entrada'); end;

maxiter = maxiter - 1;

if (maxiter == 0), error('Maximo de iteraciones alcanzado'); end;

% Usando el algoritmo de integracin compuesta podemos
% hacer las dos integrales

solsimple = intcomp(f,a,b,1,metodo);
solcomp = intcomp(f,a,b,2,metodo);

err = abs((solsimple - solcomp)/solcomp); % Error relativo

if (err < tol),
   sol = solcomp;
else
   c = (b+a)/2;
   sol = intadapt(f,a,c,tol,metodo,maxiter) + intadapt(f,c,b,tol,metodo,maxiter);
end;