% Metodo de interpolacion de Hermite con
% la expresion de Newton
%
% Uso: Px=hermnew(x,fx,fpx)
%
%     x ---> puntos donde se conoce la funcion
%    fx ---> valores de la funcion en dichos puntos
%    fx ---> valores de la derivada de la funcion en
%            dichos puntos
%    Px ---> polinomio de interpolacion resultante
%
% (C) - Carlos Garcia Argos, 11/VI/2000

function [Px]=hermnew(x,fx,fpx)
long=length(x); % Grado maximo que tendra el polinomio de interpolacion
dif=zeros(2*long); % Matriz que va a contener las diferencias divididas
colx=zeros(2*long,1);
% Creamos el vector con los puntos multiples
for k=1:2:2*long
   colx(k)=x((k+1)/2);
end;
for k=2:2:2*long
   colx(k)=x(k/2);
end;
% Introducimos los valores de la funcion por partida doble por cada punto
for k=1:2:2*long-1
   dif(k,1)=fx((k+1)/2);
end;
for k=2:2:2*long
   dif(k,1)=fx(k/2);
end;
% Montamos la columna de segundas diferencias divididas
for k=2:2*long
   inicf=dif(k-1,1);
   xin=colx(k-1);
   xfi=colx(k);
   if xin~=xfi
      dif(k,2)=(dif(k,2-1)-inicf)/(xfi-xin);
   else
      dif(k,2)=fpx((k)/2);
   end;
end;
% Iteracion loca
cont=3;
for col=cont:2*long
   for row=cont:2*long
      inicf=dif(row-1,col-1);
      xin=colx(row-cont+1);
      xfi=colx(row);
      dif(row,col)=(dif(row,col-1)-inicf)/(xfi-xin);
   end;
   cont=cont+1;
end;
coef=diag(dif)'; % la diagonal principal de la matriz son los coeficientes
                 % que hay que multiplicar.

Px=0;
for i=0:2*long-1
   pol=1;
   for k=1:i
      pol=conv(pol,[1 -colx(k)]);
   end;
   if i<0
      Px=polisum(Px,coef(1)*pol);   
   else
      Px=polisum(Px,coef(i+1)*pol);
   end;
end;
