% Metodos cuasi-Newton para optimizacion de funciones
%
% Uso: [sol,iter]=cnewtonop(G,x0,maxiter,modif,H0,tol,metodo)
%
% - G es la funcin del gradiente
% - modif es la modificacion cuasinewton (rango1, psb,
%   bfgs o dfp)
% - H0 es la aproximacion inicial a la Jacobiana (por defecto
%   la matriz identidad)
% - metodo es la forma de calcular el alfa (ninguno,
%   lineaex o lineaaprox)
%
% (C) - Carlos Garcia Argos, 23/VI/2003

function [xs,iter]=cnewtonop(G,x0,maxiter,modif,H0,tol,metodo)

if nargin<7
   metodo = 'ninguno';
end;

if nargin<4
   error('Faltan argumentos de entrada');
   return;
end;
if nargin<5
   H0 = eye(length(x0));
end;
if nargin<6
   tol=1e-5;
end;

switch lower(modif)
case {'rango1'}
   disp('   Formula de rango 1');
case {'psb'}
   disp('   Formula PSB');
case {'bfgs'}
   disp('   Formula BFGS');
case {'dfp'}
   disp('   Formula DFP');
otherwise
   error('Metodo para aproximar la Jacobiana desconocido');
end;

fin=0;
iter=1;
grad = feval(G,x0);
H = H0;

ds=-H*grad;

switch lower(metodo)
case {'ninguno'}
   disp('   alfa = 1');
   alpha = 1;
case {'lineaex'}
   disp('   Calcular alfa por linea exacta');
   alpha=(ds'*grad)/(ds'*H*ds);
case {'lineaaprox'}
   disp('   Calcular alfa por linea aproximada');
   error('Ups, no he implementado esto todavia');
otherwise
   error('Metodo para calcular alfa desconocido');
end;

x(:,1)=x0;
while (fin==0)&(iter<maxiter)
   x(:,iter+1)=x(:,iter)+alpha*ds;
   iter=iter+1;
   gradant = grad;
   grad = feval(G,x(:,iter));

   Hant = H;
   y = grad-gradant;
   p = -alpha*Hant*gradant;

   switch lower(modif)
   case {'rango1'}
      H = Hant+(p-Hant*y)*(p-Hant*y)'/(y'*(p-Hant*y));
   case {'psb'}
      Aant = Hant^-1;
      A = Aant + ((y-Aant*p)*p'+p*(y-Aant*p)')/(p'*p)-((y-Aant*p)'*p*p*p')/(p'*p)^2;
      H = A^-1;
   case {'bfgs'}
      H = Hant + ((p-Hant*y)*p'+p*(p-Hant*y)')/(p'*y) - ((p-Hant*y)'*y*p*p')/(p'*y)^2;
   case {'dfp'}
      H = Hant + (p*p')/(p'*y) - (Hant*y*y'*Hant)/(y'*Hant*y);
   otherwise
      error('Metodo para aproximar la Jacobiana desconocido');
   end;

   ds=-H*grad;

   if norm(ds,Inf)<tol
      fin=1;
   else
      switch lower(metodo)
      case {'ninguno'}
         alpha = 1;
      case {'lineaex'}
         alpha=(ds'*grad)/(ds'*H*ds);
      case {'lineaaprox'}
         error('Ups, no he implementado esto todavia');
      end;
   end;
end;

xs=x(:,iter);
if iter==maxiter
   disp('Se ha alcanzado el maximo de iteraciones');
else
   disp('Se ha encontrado un punto que cumple la tolerancia');
end;
