% Metodo del gradiente conjugado para optimizacion de funciones
% Optimiza una funcion del tipo:
%   q(x)=1/2*x'*A*x+x'*b
%
% Uso: [sol,iter]=gconjopt(A,b,xinicial,maxiter,modificacion,tol)
% xinicial debe ser un vector columna
%
% Admite las modificaciones de 'hestenes-stiefel',
% 'fletcher-reeves' o 'polak-ribiere'. Por defecto no
% hace modificacion.
%
% (C) - Carlos Garcia Argos, 21/VIII/2001

function [xs,iter]=gconjopt(A,b,x0,maxiter,modif,tol)

if nargin<4
   error('Faltan argumentos de entrada');
   return;
end;
if nargin<6
   tol=1e-5;
end;
if nargin<5
   modif='ninguna';
end;

r(:,1)=-b-A*x0;
G=-r(:,1);
iter=1;
x(:,1)=x0;
d(:,1)=r(:,1);
beta=0;
H=A;

switch lower(modif)
case {'ninguna'}
   disp('Sin usar modificacin')
case {'hestenes-stiefel'}
   disp('Modificacin de Hestenes-Stiefel')
case {'fletcher-reeves'}
   disp('Modificacin de Fletcher-Reeves')
case {'polak-ribiere'}
   disp('Modificacin de Polak-Ribire')
otherwise
   error('Mtodo desconocido');
end;

while (norm(r(:,iter))>tol)&(iter<maxiter)&(norm(G,2)~=0)
   alpha=-(G'*d(:,iter))/(d(:,iter)'*H*d(:,iter));
   x(:,iter+1)=x(:,iter)+alpha*d(:,iter);
   iter=iter+1;
   G=A*x(:,iter)+b;
   r(:,iter)=-G;
   switch lower(modif)
   case {'ninguna'}
      beta=(G'*A*d(:,iter-1))/(d(:,iter-1)'*A*d(:,iter-1));
   case {'hestenes-stiefel'}
      beta=(-r(:,iter)'*(r(:,iter)-r(:,iter-1)))/((r(:,iter)-r(:,iter-1))'*r(:,iter-1));
   case {'fletcher-reeves'}
      beta=norm(r(:,iter),2)^2/norm(r(:,iter-1),2)^2;
   case {'polak-ribiere'}
      beta=(r(:,iter)'*(r(:,iter)-r(:,iter-1)))/norm(r(:,iter-1),2)^2;
   end;
   d(:,iter)=r(:,iter)+beta*d(:,iter-1);
end;

xs=x(:,iter);

if iter==maxiter
   disp('Se ha alcanzado el nmero mximo de iteraciones');
else
   disp('Se ha encontrado una solucin que cumple la tolerancia');
end;