% Metodo de Newton para optimizacion de funciones
%
% Uso: [sol,iter]=newtonop(G,J,x0,maxiter,tol,metodo)
%
% - G es la funcin del gradiente
% - J es la funcin de la Jacobiana
% - metodo es la forma de calcular el alfa (ninguno,
%   lineaex o lineaaprox)
%
% (C) - Carlos Garcia Argos, 20/VI/2003

function [xs,iter]=newtonop(G,J,x0,maxiter,tol,metodo)

if nargin<5
   metodo = 'ninguno';
end;

if nargin<4
   error('Faltan argumentos de entrada');
   return;
end;
if nargin<5
   tol=1e-5;
end;

fin=0;
iter=1;
jacob = feval(J,x0);
grad = feval(G,x0);
ds=-jacob\grad;

switch lower(metodo)
case {'ninguno'}
   disp('   alfa = 1');
   alpha = 1;
case {'lineaex'}
   disp('   Calcular alfa por linea exacta');
   alpha=(ds'*grad)/(ds'*jacob*ds);
case {'lineaaprox'}
   disp('   Calcular alfa por linea aproximada');
   error('Ups, no he implementado esto todavia');
otherwise
   error('Metodo para calcular alfa desconocido');
end;

x(:,1)=x0;
while (fin==0)&(iter<maxiter)
   x(:,iter+1)=x(:,iter)+alpha*ds;
   iter=iter+1;
   jacob = feval(J,x(:,iter));
   grad = feval(G,x(:,iter));
   ds=-jacob\grad;
   if norm(ds,Inf)<tol
      fin=1;
   else
      switch lower(metodo)
      case {'ninguno'}
         alpha = 1;
      case {'lineaex'}
         alpha=(ds'*grad)/(ds'*jacob*ds);
      case {'lineaaprox'}
         error('Ups, no he implementado esto todavia');
      end;
   end;
end;

xs=x(:,iter);
if iter==maxiter
   disp('Se ha alcanzado el maximo de iteraciones');
else
   disp('Se ha encontrado un punto que cumple la tolerancia');
end;
