% Metodo del gradiente para optimizacion de funciones
% Optimiza una funcion del tipo
%  q(x)=1/2*x'*A*x+x'*c
%
% Uso: [sol,iter]=optgrad(A,c,x0,maxiter,tol)
%
% (C) - Carlos Garcia Argos, 11/VI/2000

function [xs,iter]=optgrad(A,c,x0,maxiter,tol)
if nargin<4
   error('Faltan argumentos de entrada');
   return;
end;
if nargin<5
   tol=1e-5;
end;

fin=0;
iter=1;
ds=-A*x0-c;
alpha=(ds'*ds)/(ds'*A*ds);
x(:,1)=x0;
while (fin==0)&(iter<maxiter)
   x(:,iter+1)=x(:,iter)+alpha*ds;
   iter=iter+1;
   ds=-A*x(:,iter)-c;
   if norm(ds,Inf)<tol
      fin=1;
   else
      alpha=norm(ds,2)^2/(ds'*A*ds);
   end;
end;

xs=x(:,iter);
if iter==maxiter
   disp('Se ha alcanzado el maximo de iteraciones');
else
   disp('Se ha encontrado un punto que cumple la tolerancia');
end;
