% Metodos cuasi-Newton de resolucion de
% Sistemas No Lineales
% Se puede escoger el metodo cuasi-Newton.
% Opciones: broyden, dfp y bfgs
%
% Uso: [sol,ffs,error]=scnewton(f,metodo,xs0,Hs0,maxiter,norma,tol)
%  f = matriz que contiene el sistema
%  metodo = broyden, dfp, bfgs
%  xs0 = valor inicial de la solucin
%  Hs0 = valor inicial de la jacobiana
%  maxiter = mximo nmero de iteraciones
%  norma = norma de la matriz para calcular el error
%  tol = tolerancia, por defecto 1e-5
%
% (C) - Carlos Garcia Argos, 8/VI/2000

function [xxs,ffs,ers]=scnewton(f,met,xsmenos1,Hsmenos1,iter,p,tol)
if nargin<7
   tol=1e-5;
   if nargin<3
      error('Faltan argumentos de entrada');
   end;
   if nargin<6
      p=2;
   end;
   if nargin<5
      iter=10;
   end;
   if nargin<4
       Hsmenos1=eye(length(xsmenos1));
   end;
end;
metok=strcmp(lower(met),'broyden')|strcmp(lower(met),'dfp')|strcmp(lower(met),'bfgs');
if metok
   iteraciones=0;
   fxsmenos1=feval(f,xsmenos1);
   xxs=xsmenos1';
   ffs=fxsmenos1';
   ers=NaN;
   while iteraciones < iter
      dsmenos1=-Hsmenos1*fxsmenos1;
      xs=xsmenos1+dsmenos1;
      xxs=[xxs;xs'];
      err=norm(dsmenos1,p);
      ers=[ers;err];
      if err<tol
         ffs=[ffs;feval(f,xs)'];
         return;
      end;
      xsmenos1=xs;
      iteraciones=iteraciones+1;
      ysmenos1=-fxsmenos1;
      fxsmenos1=feval(f,xs);
      ffs=[ffs;fxsmenos1'];
      ysmenos1=ysmenos1+fxsmenos1;
      Hsmenos1=feval(met,Hsmenos1,dsmenos1,ysmenos1);
   end; 
   disp('Se ha llegado al numero maximo de iteraciones');
else
   disp('El metodo seleccionado no es correcto');
   return;
end;

