% Metodo Newton de resolucion de
% Sistemas No Lineales
%
% Uso: [sol,valf,error]=snewton(f,j,xs0,met,maxiter,p,tol)
%  f = matriz del sistema
%  j = jacobiana del sistema
%  xs0 = vector solucin inicial
%  met = 'divizqda' -> divisin izquierda para resolver un sistema lineal
%        con la jacobiana. Otra cosa, evala el mtodo con las 3 diagonales
%        principales
%  maxiter = nmero mximo de iteraciones (por defecto 10)
%  p = norma de la matriz para hallar el error (por defecto 2)
%  tol = tolerancia de las soluciones (por defecto 1e-5)
%
% (C) - Carlos Garcia Argos, 20/VIII/2001

function [xxs,ffs,ers]=snewton(f,jacob,xsmenos1,met,maxiter,p,tol)
if nargin<7
   tol=1e-5;
   if nargin<3
      error('Faltan argumentos de entrada');
   end;
   if nargin<6
      p=2;
   end;
   if nargin<5
      maxiter=10;
   end;
   if nargin<4
      met='divizqda';
   end;
end;
xxs=xsmenos1';
ffs=[];
ers=NaN;
error=Inf;
iteraciones=0;
while (iteraciones<maxiter)&(error>tol)
   fxsmenos1=feval(f,xsmenos1);
   ffs=[ffs;fxsmenos1'];
   if strcmp(lower(met),'divizqda')
      Jfxsmenos1=feval(jacob,xsmenos1);
      xs=xsmenos1-Jfxsmenos1\fxsmenos1;
   else
      [dinf,dpri,dsup]=feval(jacob,xsmenos1);
      xs=xsmenos1-feval(met,dinf,dpri,dsup,fxsmenos1);
   end;
   xxs=[xxs;xs'];
   error=norm(xs-xsmenos1,p);
   ers=[ers;error];
   if error<tol
      ffs=[ffs;feval(f,xs)'];
   end;
   xsmenos1=xs;
   iteraciones=iteraciones+1;
end;
if iteraciones==maxiter
   disp('Se ha llegado al numero maximo de iteraciones');
end;

