% Mtodo de Gauss para Sistemas de Ecuaciones Lineales
%
% Uso: [x] = gauss(A, b, tipopivotaje)
%
% tipopivotaje puede ser 'no', 'parcial' o 'total'
% Si no se especifica se supone 'no' (sin pivotaje)
% Con el pivotaje se puede ganar en estabilidad del metodo
% 
%
% (C) - Carlos Garca Argos, 3/III/2002
%

function [x] = gauss(A,b,pivotaje)

if nargin<3
   pivotaje = 'no';
   if nargin<2
      error('Faltan argumentos de entrada');
   end;
end;

% Es vlido el sistema?
[M,N] = size(A);
tam = length(b);
if (M ~= N)|(tam ~= M)
   error('Las dimensiones del sistema no son vlidas');
end;

ordensolucion = 1:length(b);  % Orden de las soluciones (puede cambiar con pivotaje total)
for k = 1:tam,
   j = k;
   if (A(k,k) == 0)&(strcmp(pivotaje,'no') == 1)
      pivote = pivotemax(A,j,'parcial'); % Si el trmino a(k,k) es cero, pivotamos
   end;
   pivote = pivotemax(A,j,pivotaje);
   if (pivote(1) ~= j)|(pivote(2) ~= k)
      coltemp = A(:,k);  % Guardamos la columna a intercambiar
      A(:,k) = A(:,pivote(2)); % Y hacemos el intercambio
      A(:,pivote(2)) = coltemp;
      filatemp = A(j,:); % Y ahora las filas
      A(j,:) = A(pivote(1),:);
      A(pivote(1),:) = filatemp;
      
      btemp = b(j);        % Y el trmino independiente
      b(j) = b(pivote(1));
      b(pivote(1)) = btemp;
      
      soltemp = k;  % Intercambiar las soluciones tambin
      ordensolucion(k) = ordensolucion(pivote(2));
      ordensolucion(pivote(2)) = soltemp;
   end;
   
   for j = k+1:tam,
      if (A(j,k) ~= 0)
         alfa = -A(j,k)/A(k,k);
         filatemp = A(k,:)*alfa;
         A(j,:) = A(j,:) + filatemp;
         btemp = b(k)*alfa;
         b(j) = btemp + b(j);
      end;
   end;
end;

% Matriz A triangular - > sustitucion regresiva para obtener la solucion
xtemp = sustreg(A, b);

% Ahora se ordenan las soluciones
x = zeros(tam, 1);

for k = 1:tam,
    x(ordensolucion(k)) = xtemp(k);
end;
