% Metodo de Crout para matrices tridiagonales
%
% Uso: [sol iter]=gstridi(diaginf,diagsup,diagprin,b,maxiter,x0,tol)
%
%  - sol: vector solucin
%  - iter: iteraciones requeridas
%
%  - diaginf: diagonal inferior
%  - diagsup: diagonal superior
%  - diagprin: diagonal principal
%  - b: vector de trminos independientes
%  - maxiter: nmero mximo de iteraciones del mtodo
%  - x0: primer vector solucin
%  - tol: tolerancia del resultado
%
% Se pasan las diagonales como vectores
%
% (C) - Carlos Garcia Argos, 11/VI/2000

function [xs,s]=gstridi(dinf,dsup,dpri,b,maxiter,x0,tol)

   n=length(dpri);
   xs=zeros(n,1);
   s=1;
   fin=0;
   xsant=x0;
   while (s<=maxiter)&(fin==0) % 2 condiciones: mximo de iteraciones alcanzado y cumplir la tolerancia
      xs(1)=(-dsup(1)*xsant(2)+b(1))/dpri(1);
      for k=2:n-1
         xs(k)=(-dsup(k)*xsant(k+1)-dinf(k-1)*xs(k-1)+b(k))/dpri(k);
      end;
      xs(n)=(-dinf(n-1)*xs(n-1)+b(n))/dpri(n);
      if norm(xs-xsant,2)<tol  % Si cumple la tolerancia no seguimos (norma 2)
         fin=1;
      else
         s=s+1;                % Y si no, incrementamos el contador y seguimos
         xsant=xs;
      end;
   end;
   if s>maxiter
      s = s - 1;
      disp('Se ha llegado al maximo de iteraciones');
   end;
