% Funcin que resuelve un sistema lineal por el metodo
% iterativo de Richardson
%
% Uso: [solucion,paso]=richards(A,b,xinicial,gamma,maxiter)
% xinicial debe ser un vector columna
% gamma distinto de cero
%
% Se comprueba que la matriz sea cuadrada, que el sistema sea valido
% y que no se ha acabado antes de llegar al numero de iteraciones.
% No trata con errores (todava :-D)
%
% (C) - Carlos Garca Argos, 26/III/2001

function [X,paso]=richards(A,b,x0,gamma,iter)


   % Comprobaciones iniciales
   if nargin<5, 'Faltan argumentos de entrada'
      return;
   end;
   [cols filas]=size(A);
   if cols~=filas
      disp('La matriz no es cuadrada');
      return;
   end;
   if (cols~=size(x0))|(cols~=size(b))
      disp('El sistema no es valido');
      return;
   end;
   if gamma==0
      disp('El parmetro gamma no puede ser cero');
      return;
   end;
   
   % Primera iteracin
   paso=1;
   fin=0;
   x(:,1)=x0+gamma*(b-A*x0);
   while (paso<iter)&(fin==0)
      paso=paso+1;
      x(:,paso)=x(:,paso-1)+gamma*(b-A*x(:,paso-1));
      if x(:,paso)==x(:,paso-1)
         fin=1;
      end;
   end;
   X=x(:,paso);

